import { SeverityLevel } from '@salesforce/code-analyzer-core';
import { CodeAnalyzerConfigFactory } from '../factories/CodeAnalyzerConfigFactory';
import { EnginePluginsFactory } from '../factories/EnginePluginsFactory';
import { ResultsViewer } from '../viewers/ResultsViewer';
import { RunSummaryViewer } from '../viewers/RunSummaryViewer';
import { ResultsWriter } from '../writers/ResultsWriter';
import { LogEventListener } from '../listeners/LogEventListener';
import { ProgressEventListener } from '../listeners/ProgressEventListener';
export type RunDependencies = {
    configFactory: CodeAnalyzerConfigFactory;
    pluginsFactory: EnginePluginsFactory;
    logEventListeners: LogEventListener[];
    progressListeners: ProgressEventListener[];
    writer: ResultsWriter;
    resultsViewer: ResultsViewer;
    runSummaryViewer: RunSummaryViewer;
};
export type RunInput = {
    'config-file'?: string;
    'output-file': string[];
    'path-start'?: string[];
    'rule-selector': string[];
    'severity-threshold'?: SeverityLevel;
    workspace: string[];
};
export declare class RunAction {
    private readonly dependencies;
    private constructor();
    execute(input: RunInput): Promise<void>;
    static createAction(dependencies: RunDependencies): RunAction;
}
