"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnnotatedConfigModel = exports.OutputFormat = void 0;
const yaml = __importStar(require("js-yaml"));
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
var OutputFormat;
(function (OutputFormat) {
    OutputFormat["RAW_YAML"] = "RAW_YAML";
    OutputFormat["STYLED_YAML"] = "STYLED_YAML";
})(OutputFormat || (exports.OutputFormat = OutputFormat = {}));
class AnnotatedConfigModel {
    relevantEngines;
    userContext;
    defaultContext;
    constructor(relevantEngines, userContext, defaultContext) {
        this.relevantEngines = relevantEngines;
        this.userContext = userContext;
        this.defaultContext = defaultContext;
    }
    toFormattedOutput(format) {
        // istanbul ignore else: Should be impossible
        if (format === OutputFormat.STYLED_YAML) {
            return toYaml(this.relevantEngines, this.userContext, this.defaultContext, true);
        }
        else if (format === OutputFormat.RAW_YAML) {
            return toYaml(this.relevantEngines, this.userContext, this.defaultContext, false);
        }
        else {
            throw new Error(`Unsupported`);
        }
    }
    static fromSelection(relevantEngines, userContext, defaultContext) {
        return new AnnotatedConfigModel(relevantEngines, userContext, defaultContext);
    }
}
exports.AnnotatedConfigModel = AnnotatedConfigModel;
function toYaml(relevantEngines, userContext, defaultContext, styled) {
    let result = '';
    // First, add the header.
    const topLevelDescription = code_analyzer_core_1.CodeAnalyzerConfig.getConfigDescription();
    result += `${toYamlComment(getYamlSectionWrapper(), styled)}\n`;
    result += `${toYamlComment(topLevelDescription.overview, styled)}\n`;
    result += `${toYamlComment(getYamlSectionWrapper(), styled)}\n`;
    result += '\n';
    // Next add `config_root`
    result += `${toYamlComment(topLevelDescription.fieldDescriptions.config_root, styled)}\n`;
    result += `config_root: ${toYamlWithDerivedValueComment(userContext.config.getConfigRoot(), process.cwd(), styled)}\n`;
    result += '\n';
    // Then `log_folder'
    result += `${toYamlComment(topLevelDescription.fieldDescriptions.log_folder, styled)}\n`;
    result += `log_folder: ${toYamlWithDerivedValueComment(userContext.config.getLogFolder(), defaultContext.config.getLogFolder(), styled)}\n`;
    result += '\n';
    // Then the `rules` section
    result += `${toYamlComment(topLevelDescription.fieldDescriptions.rules, styled)}\n`;
    result += `${toYamlRules(userContext, defaultContext, styled)}\n`;
    result += '\n';
    // Then the `engines` section
    result += `${toYamlComment(topLevelDescription.fieldDescriptions.engines, styled)}\n`;
    result += `${toYamlEngines(relevantEngines, userContext, defaultContext, styled)}`;
    return result;
}
function getYamlSectionWrapper() {
    return '='.repeat(70);
}
function toYamlComment(comment, styled, indentLength = 0) {
    // At the start of the string, and at the start of every line...
    return comment.replace(/^.*/gm, s => {
        // ...Inject a `# ` at the start to make it a comment...
        let commented = `# ${s}`;
        // ...Apply styling if requested...
        commented = styled ? (0, StylingUtil_1.makeGrey)(commented) : commented;
        // ...And indent to the requested amount
        return (0, StylingUtil_1.indent)(commented, indentLength);
    });
}
function toYamlWithDerivedValueComment(userValue, defaultValue, styled) {
    if (userValue == null || userValue === defaultValue) {
        const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.last-calculated-as', [defaultValue]);
        return `null ${toYamlComment(comment, styled)}`;
    }
    else {
        return `${userValue}`;
    }
}
function toYamlRules(userContext, defaultContext, styled) {
    if (userContext.rules.getCount() === 0) {
        const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-rules-selected');
        return `rules: {} ${toYamlComment(comment, styled)}`;
    }
    let results = 'rules:\n';
    for (const engineName of userContext.rules.getEngineNames()) {
        results += `\n${toYamlComment(getYamlSectionWrapper(), styled, 2)}\n`;
        results += `${toYamlComment((0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.rule-overrides-section', [engineName.toUpperCase()]), styled, 2)}\n`;
        results += `${toYamlComment(getYamlSectionWrapper(), styled, 2)}\n`;
        results += (0, StylingUtil_1.indent)(`${engineName}:`, 2) + '\n';
        for (const userRule of userContext.rules.getRulesFor(engineName)) {
            const defaultRule = getDefaultRule(defaultContext, engineName, userRule.getName());
            results += (0, StylingUtil_1.indent)(toYamlRule(userRule, defaultRule, styled), 4);
        }
    }
    return results;
}
function getDefaultRule(defaultContext, engineName, ruleName) {
    try {
        return defaultContext.rules.getRule(engineName, ruleName);
    }
    catch (e) {
        // istanbul ignore next
        return null;
    }
}
function toYamlRule(userRule, defaultRule, styled) {
    const ruleName = userRule.getName();
    const userSeverity = userRule.getSeverityLevel();
    const userTags = userRule.getTags();
    let severityYaml = yaml.dump({ severity: userSeverity });
    let tagsYaml = yaml.dump({ tags: userTags });
    if (defaultRule != null) {
        const defaultSeverity = defaultRule.getSeverityLevel();
        const defaultTagsJson = JSON.stringify(defaultRule.getTags());
        if (userSeverity !== defaultSeverity) {
            const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.modified-from', [defaultSeverity]);
            severityYaml = severityYaml.replace('\n', ` ${toYamlComment(comment, styled)}\n`);
        }
        if (JSON.stringify(userTags) !== defaultTagsJson) {
            const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.modified-from', [defaultTagsJson]);
            // The YAML spec requires a trailing newline, so we know that there's at least one newline somewhere in the
            // string. If we inject a comment before the first newline we encounter, then it will look clean.
            tagsYaml = tagsYaml.replace('\n', ` ${toYamlComment(comment, styled)}\n`);
        }
    }
    return `"${ruleName}":\n${(0, StylingUtil_1.indent)(severityYaml, 2)}${(0, StylingUtil_1.indent)(tagsYaml, 2)}`;
}
function toYamlEngines(relevantEngines, userContext, defaultContext, styled) {
    if (relevantEngines.size === 0) {
        const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-engines-selected');
        return `engines: {} ${toYamlComment(comment, styled)}`;
    }
    let results = 'engines:\n';
    for (const engineName of relevantEngines.keys()) {
        const engineConfigDescriptor = userContext.core.getEngineConfigDescription(engineName);
        const userEngineConfig = userContext.core.getEngineConfig(engineName);
        const defaultEngineConfig = defaultContext.core.getEngineConfig(engineName);
        results += `\n${toYamlComment(getYamlSectionWrapper(), styled, 2)}\n`;
        // Engines are guaranteed to have an overview, even if it's just generic text.
        results += `${toYamlComment(engineConfigDescriptor.overview, styled, 2)}\n`;
        results += `${toYamlComment(getYamlSectionWrapper(), styled, 2)}\n`;
        results += (0, StylingUtil_1.indent)(`${engineName}:`, 2) + '\n';
        // By fiat, the field description will always include, at minimum, an entry for "disable_engine", so we can
        // assume that the object is not undefined.
        for (const configField of Object.keys(engineConfigDescriptor.fieldDescriptions)) {
            const fieldDescription = engineConfigDescriptor.fieldDescriptions[configField];
            const defaultFieldValue = defaultEngineConfig[configField] ?? null;
            const userFieldValue = userEngineConfig[configField] ?? defaultFieldValue;
            // Add a leading newline to visually break up the property from the previous one.
            results += '\n' + toYamlComment(fieldDescription, styled, 4) + '\n';
            let fieldYaml = (0, StylingUtil_1.indent)(`${yaml.dump({ [configField]: userFieldValue })}`, 4);
            if (JSON.stringify(userFieldValue) !== JSON.stringify(defaultFieldValue)) {
                const comment = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.modified-from', [JSON.stringify(defaultFieldValue)]);
                fieldYaml = fieldYaml.replace('\n', ` ${toYamlComment(comment, styled)}\n`);
            }
            results += fieldYaml;
        }
    }
    return results;
}
//# sourceMappingURL=ConfigModel.js.map