"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunSummaryDisplayer = void 0;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
class RunSummaryDisplayer {
    display;
    constructor(display) {
        this.display = display;
    }
    view(results, config, outfiles) {
        this.display.displayLog((0, StylingUtil_1.toStyledHeader)((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.header')));
        // Use empty line as a visual separator
        this.display.displayLog('');
        if (results.getViolationCount() === 0) {
            this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.found-no-violations'));
        }
        else {
            this.displayResultsSummary(results);
        }
        // Use empty line as a visual separator
        this.display.displayLog('');
        if (outfiles.length === 0) {
            this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.no-outfiles'));
        }
        else {
            this.displayOutfiles(outfiles);
        }
        // Use empty line as a visual separator
        this.display.displayLog('');
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.log-file-location'));
        this.display.displayLog((0, StylingUtil_1.indent)(config.getLogFolder()));
    }
    displayResultsSummary(results) {
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.violations-total', [results.getViolationCount()]));
        for (const sev of Object.values(code_analyzer_core_1.SeverityLevel)) {
            // Some of the keys will be numbers, since the enum is numerical. Skip those.
            if (typeof sev !== 'string') {
                continue;
            }
            const sevCount = results.getViolationCountOfSeverity(code_analyzer_core_1.SeverityLevel[sev]);
            if (sevCount > 0) {
                this.display.displayLog((0, StylingUtil_1.indent)((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.violations-item', [sevCount, sev])));
            }
        }
    }
    displayOutfiles(outfiles) {
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.RunSummaryViewer, 'summary.outfiles-total'));
        for (const outfile of outfiles) {
            this.display.displayLog((0, StylingUtil_1.indent)(outfile));
        }
    }
}
exports.RunSummaryDisplayer = RunSummaryDisplayer;
//# sourceMappingURL=RunSummaryViewer.js.map